@@ -119,6 +119,8 @@ class GenericHandler(tornado.web.RequestHandler):
         logging.debug('Received request:\n%s' % self.request.__dict__)
 
     def add_params(self, context):
+        if not hasattr(self, 'custom_params'):
+            return context
         for key, param in self.custom_params.items():
             if isinstance(param, PathParam):
                 context[key] = _safe_path_split(self.request.path)[param.index]
@@ -129,34 +131,29 @@ class GenericHandler(tornado.web.RequestHandler):
         return context
 
     def render_template(self):
-        source_text = None
-
-        is_response_str = isinstance(self.custom_response, str)
+        is_binary = False
         template_engine = _detect_engine(self.custom_response, 'response', default=self.definition_engine)
+        source_text = self.custom_response['body'] if 'body' in self.custom_response else None
 
-        if is_response_str:
-            source_text = self.custom_response
-        elif not self.custom_response:
-            source_text = ''
-            is_response_str = True
-        elif 'body' in self.custom_response:
-            source_text = self.custom_response['body']
-            is_response_str = True
-        else:
-            return ''
+        if source_text is None:
+            return source_text
 
         if len(source_text) > 1 and source_text[0] == '@':
             template_path = self.resolve_relative_path(source_text)
             if template_path is None:
                 return None
-            with open(template_path, 'r') as file:
-                logging.info('Reading template file from path: %s' % template_path)
+            with open(template_path, 'rb') as file:
+                logging.info('Reading external file from path: %s' % template_path)
                 source_text = file.read()
-                logging.debug('Template file text: %s' % source_text)
-            is_response_str = False
+                try:
+                    source_text = source_text.decode('utf-8')
+                    logging.debug('Template file text: %s' % source_text)
+                except UnicodeDecodeError:
+                    is_binary = True
+                    logging.debug('Template file is binary. Templating disabled.')
 
         compiled = None
-        if 'useTemplating' in self.custom_response and self.custom_response['useTemplating'] is False:
+        if is_binary or not self.custom_response.get('useTemplating', True):
             compiled = source_text
         else:
             if template_engine == PYBARS:
@@ -179,7 +176,8 @@ class GenericHandler(tornado.web.RequestHandler):
             )
             compiled, _ = renderer.render()
 
-        logging.debug('Render output: %s' % compiled)
+        if not is_binary:
+            logging.debug('Render output: %s' % compiled)
 
         return compiled
 
@@ -295,7 +293,7 @@ class GenericHandler(tornado.web.RequestHandler):
             for key, value in self.globals['headers'].items():
                 self.set_header(key, value)
 
-        if not isinstance(self.custom_response, dict) or 'headers' not in self.custom_response:
+        if 'headers' not in self.custom_response:
             return
 
         for key, value in self.custom_response['headers'].items():
@@ -408,11 +406,37 @@ class GenericHandler(tornado.web.RequestHandler):
 
             if self.should_cors():
                 self.respond_cors()
-            else:
-                self.set_cors_headers()
 
             _id = alternative['id']
-            response = alternative['response']
+            if 'response' in alternative:
+                response = alternative['response']
+                if isinstance(alternative['response'], list):
+                    if not len(alternative['response']) > 0:
+                        response = {
+                            'body': None
+                        }
+                    else:
+                        if 'multiResponsesIndex' not in alternative:
+                            alternative['multiResponsesIndex'] = 0
+                        else:
+                            alternative['multiResponsesIndex'] += 1
+
+                        if alternative['multiResponsesIndex'] > len(alternative['response']) - 1:
+                            if alternative.get('multiResponsesLooped', True):
+                                alternative['multiResponsesIndex'] = 0
+                            else:
+                                self.set_status(410)
+                                self.finish()
+                                return
+                        response = alternative['response'][alternative['multiResponsesIndex']]
+
+                response = response if isinstance(response, dict) else {
+                    'body': response
+                }
+            else:
+                response = {
+                    'body': None
+                }
             params = alternative['params']
             context = alternative['context']
             return _id, response, params, context
@@ -434,9 +458,7 @@ class GenericHandler(tornado.web.RequestHandler):
         super().finish(chunk)
 
     def should_write(self):
-        return not hasattr(self, 'custom_response') or (
-            'body' in self.custom_response or isinstance(self.custom_response, str)
-        )
+        return not hasattr(self, 'custom_response') or 'body' in self.custom_response
 
     def decoder(self, string):
         try:
@@ -447,6 +469,16 @@ class GenericHandler(tornado.web.RequestHandler):
     def resolve_relative_path(self, source_text):
         relative_path = None
         orig_relative_path = source_text[1:]
+
+        renderer = TemplateRenderer(
+            self.definition_engine,
+            orig_relative_path,
+            inject_objects=self.custom_context,
+            inject_methods=[],
+            add_params_callback=self.add_params
+        )
+        orig_relative_path, _ = renderer.render()
+
         error_msg = 'External template file \'%s\' couldn\'t be accessed or found!' % orig_relative_path
         if orig_relative_path[0] == '/':
             orig_relative_path = orig_relative_path[1:]
@@ -471,9 +503,8 @@ class GenericHandler(tornado.web.RequestHandler):
         if ORIGIN not in self.request.headers._dict:
             # Invalid CORS preflight request
             self.set_status(404)
-            self.finish()
+            return
 
-        self.set_cors_headers()
         self.set_status(204)
         self.finish()
 
@@ -492,6 +523,7 @@ class GenericHandler(tornado.web.RequestHandler):
             PROGRAM.capitalize(),
             mockintosh.__version__
         ))
+        self.set_cors_headers()
 
     def should_cors(self):
         return not self.__class__.__name__ == 'ErrorHandler' and (
