@@ -28,10 +28,18 @@ class IntegrationTests(unittest.TestCase):
         resp = requests.get(SRV1 + '/')
         self.assertEqual(200, resp.status_code)
 
+        resp = requests.get(SRV1 + '/response-not-required')
+        self.assertEqual(200, resp.status_code)
+
         # since service :8001 does not specify hostname, it should accept any
         resp = requests.get(SRV1 + '/', headers={'Host': 'someservice.domain'})
         self.assertEqual(200, resp.status_code)
 
+        resp = requests.get(SRV1 + '/404-img.png')
+        self.assertEqual(404, resp.status_code)
+        self.assertEqual(b"\x89\x50\x4E\x47\x0D\x0A\x1A\x0A", resp.content[:8])
+        self.assertEqual("image/png", resp.headers.get("content-type"))
+
     def test_host_header(self):
         resp = requests.get(SRV2 + '/')
         self.assertEqual(404, resp.status_code)
@@ -141,6 +149,9 @@ class IntegrationTests(unittest.TestCase):
         self.assertEqual(200, resp.status_code)
         self.assertIn("<title>CORS Example</title>", resp.text)
 
+        resp = requests.get(SRV1 + '/parameterized3/image.png')
+        self.assertEqual(200, resp.status_code)
+
         resp = requests.get(SRV1 + '/insecure-configuration1')
         self.assertEqual(403, resp.status_code)
 
@@ -174,3 +185,32 @@ class IntegrationTests(unittest.TestCase):
 
         resp = requests.options(SRV1 + '/cors-request')
         self.assertEqual(404, resp.status_code)  # maybe it should be 400
+
+    def test_multiresponse(self):
+        resp = requests.get(SRV1 + '/multi-response-looped')
+        self.assertEqual(200, resp.status_code)
+        self.assertIn("<title>CORS Example</title>", resp.text)
+
+        resp = requests.get(SRV1 + '/multi-response-looped')
+        self.assertEqual(200, resp.status_code)
+        self.assertEqual("image/png", resp.headers.get("content-type"))
+
+        resp = requests.get(SRV1 + '/multi-response-looped')
+        self.assertEqual(200, resp.status_code)
+        self.assertIn("just some text", resp.text)
+
+        resp = requests.get(SRV1 + '/multi-response-looped')
+        self.assertEqual(200, resp.status_code)
+        self.assertIn("<title>CORS Example</title>", resp.text)
+
+    def test_multiresponse_noloop(self):
+        resp = requests.get(SRV1 + '/multi-response-nonlooped')
+        self.assertEqual(200, resp.status_code)
+        self.assertIn("resp1", resp.text)
+
+        resp = requests.get(SRV1 + '/multi-response-nonlooped')
+        self.assertEqual(200, resp.status_code)
+        self.assertIn("resp2", resp.text)
+
+        resp = requests.get(SRV1 + '/multi-response-nonlooped')
+        self.assertEqual(410, resp.status_code)
